from bs4 import BeautifulSoup
import csv
import datetime
import json
from shutil import copyfile
import sys
import requests
import time
import urllib.request

import config

PROGRAM_DESCRIPTION_LEN_LIMIT = 50

session = requests.Session()
wikipedia_links_cache = dict()


def french_day():
    name_day = ["lundi", "mardi", "mercredi", "jeudi", "vendredi", "samedi", "dimanche"]

    return name_day[time.localtime().tm_wday]


def replace_image_size(img_url):
    return img_url.replace("/64x90/", "/{}x{}/".format(config.CAPTION_WIDTH, config.CAPTION_HEIGHT))


def number_programs(d):
    return len(d.select(".mainBroadcastCard"))


def get_img_src(item):
    if len(item) == 0:
        return None

    if "lazyload" in item[0]['class']:
        return item[0]['data-src']
    else:
        return item[0]['src']


def parse_program(d):
    program = {
        'title': d.select(".mainBroadcastCard-title a")[0].string.strip(),
        'hour': d.select('.mainBroadcastCard-startingHour')[0].string.strip(),
        'episode': "",
        'full_episode': "",
        'type': d.select(".mainBroadcastCard-format")[0].string.strip(),
        'image': get_img_src(d.select(".mainBroadcastCard-imageContent img")),
        'wikipedia_links': None
    }

    if len(d.select('.mainBroadcastCard-subtitle')) > 0:
        program['episode'] = d.select('.mainBroadcastCard-subtitle')[0].string.strip()
        program['full_episode'] = program['episode']

        if len(program['episode']) > PROGRAM_DESCRIPTION_LEN_LIMIT:
            program['episode'] = program['episode'][:PROGRAM_DESCRIPTION_LEN_LIMIT].strip() + "..."

    if config.WIKIPEDIA_LINKS and program['type'].split(" ")[0] in ['Série', 'Cinéma', 'Téléfilm', 'Comédie', 'Western', 'Film', 'Drame', 'Thriller']:
        if program['title'] not in wikipedia_links_cache:
            req = session.get(url="https://fr.wikipedia.org/w/api.php", params={
                "action": "opensearch",
                "namespace": "0",
                "search": program['title'],
                "limit": "5",
                "format": "json"
            })
            data = req.json()
            if len(data[1]) > 0:
                wikipedia_links_cache[program['title']] = {data[1][i]: data[3][i] for i in range(len(data[1]))}
            else:
                wikipedia_links_cache[program['title']] = None

        if wikipedia_links_cache[program['title']] is not None:
            program['wikipedia_links'] = wikipedia_links_cache[program['title']]

    return program


def write_program(p):
    html = ""
    if p['image'] is not None:
        html += '\t\t\t\t\t\t\t<img class="float-start img-rounded img-margin" src="{}">\n'.format(replace_image_size(p['image']))
    else:
        html += '\t\t\t\t\t\t\t<div class="float-start" style="margin-right: 5px; width: {}px; height: {}px;"></div>\n'.format(config.CAPTION_WIDTH, config.CAPTION_HEIGHT)
    html += "\t\t\t\t\t\t\t<small>\n"
    html += "\t\t\t\t\t\t\t\t{} <em>{}</em>\n".format(p['hour'], p['type'])
    if p['wikipedia_links'] is not None:
        html += '\t\t\t\t\t\t\t\t<a tabindex="-1" role="button" class="btn btn-sm btn-light" data-bs-toggle="popover">\n'
        html += '\t\t\t\t\t\t\t\t\t<i class="fab fa-wikipedia-w"></i>\n'
        html += '\t\t\t\t\t\t\t\t</a>\n'
        html += '\t\t\t\t\t\t\t\t<div class="list-group popover-content d-none">\n'
        for page in p['wikipedia_links']:
            html += '\t\t\t\t\t\t\t\t\t<a href="{}" target="_blank" class="list-group-item list-group-item-action">{}</a>\n'.format(p['wikipedia_links'][page], page)
        html += '\t\t\t\t\t\t\t\t</div>\n'
    html += "\t\t\t\t\t\t\t</small>\n"
    html += "\t\t\t\t\t\t\t<h4>{}</h4>\n".format(p['title'])
    html += '\t\t\t\t\t\t\t<span class="d-none">{}</span><span>{}</span>\n'.format(p['full_episode'], p['episode'])

    return html


def get_channel_number(s):
    return int(s[s.find('°')+1:])


def get_channels(url, tee_csv=False):
    # Use curl user-agent, because server requires from other user-agent they
    # present a cookie containing a token acknowledging acceptance of being
    # tracked...
    req = urllib.request.Request(url, headers={'User-Agent': 'curl/7.81.0'})
    soup = BeautifulSoup(urllib.request.urlopen(req).read(), "html.parser")
    # For tests, download the source page and load it locally:
    # soup = BeautifulSoup(open('src.html'), "html.parser")

    channels = []
    c = 0

    gridrow_cards = soup.select(".gridRow-cards")

    while c < len(gridrow_cards) and (len(channels) == 0 or channels[-1]["number"] < 26):
        d = gridrow_cards[c]
        channel = {
            # 'name': list(d.select(".bouquet-cardsChannelItemLink")[0].strings)[-1].strip(),
            'logo': get_img_src(d.select(".pictureTagGenerator img")),
            'number': get_channel_number(d.select(".gridRow-cardsChannelNumber")[0].string.strip()),
            'programs': []
        }

        for i in range(min(number_programs(d), 2)):
            channel['programs'].append(parse_program(d.select(".mainBroadcastCard")[i]))

        channels.append(channel)
        c += 1

    if len(channels) == 0:
        raise Exception("No channel found.")

    html = "\t\t\t\t<table class=\"table table-striped\">\n"
    csv_writer = csv.writer(sys.stdout)

    for c in channels:
        if c['number'] not in config.IGNORE_CHANNELS:
            html += "\t\t\t\t\t<tr>\n"
            html += "\t\t\t\t\t\t<td class=\"text-center\">\n"
            html += "\t\t\t\t\t\t\t<h6>{}</h6>\n".format(c['number'])
            html += "\t\t\t\t\t\t\t<img src=\"{}\">\n".format(c['logo'])
            html += "\t\t\t\t\t\t</td>\n"

            num_programs = 0
            for p in range(len(c['programs'])):
                html += '\t\t\t\t\t\t<td class="program">\n'
                html += write_program(c['programs'][p])
                html += "\t\t\t\t\t\t</td>\n"
                num_programs += 1

                if tee_csv:
                    csv_writer.writerow([
                        str(datetime.date.today()),
                        c['number'],
                        c['programs'][p]['hour'],
                        c['programs'][p]['title'],
                        c['programs'][p]['full_episode'],
                        c['programs'][p]['type']
                    ])

            if num_programs < 2:
                html += "\t\t\t\t\t\t<td>\n"
                html += "\t\t\t\t\t\t</td>\n"

            html += "\t\t\t\t\t</tr>\n"


    html += "\t\t\t\t</table>\n"

    return html


copyfile(config.HEADER_FILE, config.INDEX_FILE)

with open(config.INDEX_FILE, 'a') as file, open(config.FOOTER_FILE, 'r') as footer:
    file.write("ce {} soir</button></li>\n".format(french_day()))
    if config.GET_TOMORROW:
        file.write('\t\t\t<li role="presentation" class="nav-item"><button class="nav-link" id="tomorrow-tab" data-bs-toggle="tab" data-bs-target="#tomorrow" role="tab">demain soir</button></li>\n')
    file.write("\t\t</ul>\n")
    file.write('\t\t<div class="tab-content">\n')

    file.write('\t\t\t<div role="tabpanel" class="tab-pane show active" id="today">\n')
    file.write(get_channels(config.SRC_URL, tee_csv=True))
    file.write("\t\t\t</div>\n")

    if config.GET_TOMORROW:
        file.write('\t\t\t<div role="tabpanel" class="tab-pane" id="tomorrow">\n')
        tomorrow_date = datetime.date.today() + datetime.timedelta(days=1)
        file.write(get_channels(config.SRC_TOMORROW_URL + tomorrow_date.strftime("%Y-%m-%d")))

    footer_html = footer.readlines()
    file.writelines(footer_html)

if config.HEALTHCHECK:
    urllib.request.urlopen(config.HEALTHCHECK_URL)
